
import { GET, POST } from '../../../libs/request';
import { API_HOST } from '../../../libs/config';
import Humps from './humps.js';
import iconfSwitch from '../../../libs/switch';
import md5 from '../../../vendors/md5';

//注：g_sourceApiArry与g_toJavaApiArry的length要相等，且一一对应
const g_sourceApiArry = [ '/list', '/content', '/list/recommend', '/code/recent', '/list/mine', '/code/gain', '/code/mine','/yohood/content' ];
const g_toJavaApiArry = [ 'app.yoluck.activityList', 'app.yoluck.getContent', 'app.yoluck.recommendList', 'app.yoluck.recent', 'app.yoluck.participationList', 'app.yoluck.getCode', 'app.yoluck.userCode','app.yohood.getContent' ];


const app = getApp();

class Service {
  constructor(url = '') {
    this.url = url;
  }

  _get(path, data) {
    let method = path ? this.url + path : API_HOST;
    let pathIndex = g_sourceApiArry.indexOf(path);

    let executeReq = () => {
      //适配服务器更换Java接口，开关控制是否切换
      if (app.globalData.switch_javaApiEnable && pathIndex >= 0 && pathIndex < g_toJavaApiArry.length) {
        method = API_HOST;
        data.method = g_toJavaApiArry[pathIndex];
      }

      return GET(method, data).then(result => {

        //适配服务器更换Java接口，开关控制是否切换
        if (app.globalData.switch_javaApiEnable && pathIndex >= 0) {
          if (result.data&& result.data.list) {
            let newList = Humps.decamelizeKeys(Object.values(result.data.list));
            result.data.list = newList;
          } else {
            let newData = Humps.decamelizeKeys(result.data);
            result.data = newData;
          }
        }
        return result
      });
    };
    
    let needSwitch = g_sourceApiArry.indexOf(path) > -1;

    if (!app.globalData.hasSetSwitch && needSwitch) {
      return iconfSwitch().then(data => {
        app.globalData.hasSetSwitch = true;
        if (data['javaapi'] && '0' === data['javaapi']) {
          app.globalData.switch_javaApiEnable = false;
        }
        return executeReq();
      }, () => {
        app.globalData.hasSetSwitch = true;
        return executeReq();
      });
    } else {
      return executeReq();
    }
  }

  _post(path, data) {
    let method = path ? this.url + path : API_HOST;
    
    return POST(method, data).then(result => {
      return result;
    });
  }
}

export default Service;